// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.Azure.Management.Sql;
using Microsoft.Azure.Management.Sql.Models;

namespace Microsoft.Azure.Management.Sql
{
    /// <summary>
    /// The Windows Azure SQL Database management API provides a RESTful set of
    /// web services that interact with Windows Azure SQL Database services to
    /// manage your databases. The API enables users to create, retrieve,
    /// update, and delete databases and servers.
    /// </summary>
    public static partial class RecommendedElasticPoolOperationsExtensions
    {
        /// <summary>
        /// Returns information about an Azure SQL Recommended Elastic Pool.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Sql.IRecommendedElasticPoolOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the Resource Group to which the Azure SQL
        /// Recommended Server belongs.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Database Server for which the
        /// Azure SQL Recommended Elastic Pool is.
        /// </param>
        /// <param name='recommendedElasticPoolName'>
        /// Required. The name of the Azure SQL Recommended Elastic Pool to be
        /// retrieved.
        /// </param>
        /// <returns>
        /// Represents the response to a Get Azure Sql Recommended Resource
        /// pool request.
        /// </returns>
        public static RecommendedElasticPoolGetResponse Get(this IRecommendedElasticPoolOperations operations, string resourceGroupName, string serverName, string recommendedElasticPoolName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IRecommendedElasticPoolOperations)s).GetAsync(resourceGroupName, serverName, recommendedElasticPoolName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Returns information about an Azure SQL Recommended Elastic Pool.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Sql.IRecommendedElasticPoolOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the Resource Group to which the Azure SQL
        /// Recommended Server belongs.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Database Server for which the
        /// Azure SQL Recommended Elastic Pool is.
        /// </param>
        /// <param name='recommendedElasticPoolName'>
        /// Required. The name of the Azure SQL Recommended Elastic Pool to be
        /// retrieved.
        /// </param>
        /// <returns>
        /// Represents the response to a Get Azure Sql Recommended Resource
        /// pool request.
        /// </returns>
        public static Task<RecommendedElasticPoolGetResponse> GetAsync(this IRecommendedElasticPoolOperations operations, string resourceGroupName, string serverName, string recommendedElasticPoolName)
        {
            return operations.GetAsync(resourceGroupName, serverName, recommendedElasticPoolName, CancellationToken.None);
        }
        
        /// <summary>
        /// Returns information about an Azure SQL Database inside of an Azure
        /// Sql Recommended Elastic Pool.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Sql.IRecommendedElasticPoolOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the Resource Group to which the server
        /// belongs.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Database Server on which the
        /// database is hosted.
        /// </param>
        /// <param name='recommendedElasticPoolName'>
        /// Required. The name of the Azure SQL Database Elastic Pool to be
        /// retrieved.
        /// </param>
        /// <param name='databaseName'>
        /// Required. The name of the Azure SQL Database to be retrieved.
        /// </param>
        /// <returns>
        /// Represents the response to a Get Azure Sql Database request.
        /// </returns>
        public static DatabaseGetResponse GetDatabases(this IRecommendedElasticPoolOperations operations, string resourceGroupName, string serverName, string recommendedElasticPoolName, string databaseName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IRecommendedElasticPoolOperations)s).GetDatabasesAsync(resourceGroupName, serverName, recommendedElasticPoolName, databaseName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Returns information about an Azure SQL Database inside of an Azure
        /// Sql Recommended Elastic Pool.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Sql.IRecommendedElasticPoolOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the Resource Group to which the server
        /// belongs.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Database Server on which the
        /// database is hosted.
        /// </param>
        /// <param name='recommendedElasticPoolName'>
        /// Required. The name of the Azure SQL Database Elastic Pool to be
        /// retrieved.
        /// </param>
        /// <param name='databaseName'>
        /// Required. The name of the Azure SQL Database to be retrieved.
        /// </param>
        /// <returns>
        /// Represents the response to a Get Azure Sql Database request.
        /// </returns>
        public static Task<DatabaseGetResponse> GetDatabasesAsync(this IRecommendedElasticPoolOperations operations, string resourceGroupName, string serverName, string recommendedElasticPoolName, string databaseName)
        {
            return operations.GetDatabasesAsync(resourceGroupName, serverName, recommendedElasticPoolName, databaseName, CancellationToken.None);
        }
        
        /// <summary>
        /// Returns information about Azure SQL Recommended Elastic Pools.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Sql.IRecommendedElasticPoolOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the Resource Group to which the Azure SQL
        /// Recommended Serve belongs.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Database Server in which Azure
        /// SQL Recommended Elastic Pools are hosted.
        /// </param>
        /// <returns>
        /// Represents the response to a List Azure Sql Recommended Elastic
        /// Pool request.
        /// </returns>
        public static RecommendedElasticPoolListResponse List(this IRecommendedElasticPoolOperations operations, string resourceGroupName, string serverName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IRecommendedElasticPoolOperations)s).ListAsync(resourceGroupName, serverName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Returns information about Azure SQL Recommended Elastic Pools.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Sql.IRecommendedElasticPoolOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the Resource Group to which the Azure SQL
        /// Recommended Serve belongs.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Database Server in which Azure
        /// SQL Recommended Elastic Pools are hosted.
        /// </param>
        /// <returns>
        /// Represents the response to a List Azure Sql Recommended Elastic
        /// Pool request.
        /// </returns>
        public static Task<RecommendedElasticPoolListResponse> ListAsync(this IRecommendedElasticPoolOperations operations, string resourceGroupName, string serverName)
        {
            return operations.ListAsync(resourceGroupName, serverName, CancellationToken.None);
        }
        
        /// <summary>
        /// Returns information about an Azure SQL Database inside of an Azure
        /// Sql Recommended Elastic Pool.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Sql.IRecommendedElasticPoolOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the Resource Group to which the server
        /// belongs.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Database Server on which the
        /// database is hosted.
        /// </param>
        /// <param name='recommendedElasticPoolName'>
        /// Required. The name of the Azure SQL Recommended Elastic Pool to be
        /// retrieved.
        /// </param>
        /// <returns>
        /// Represents the response to a List Azure Sql Database request.
        /// </returns>
        public static DatabaseListResponse ListDatabases(this IRecommendedElasticPoolOperations operations, string resourceGroupName, string serverName, string recommendedElasticPoolName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IRecommendedElasticPoolOperations)s).ListDatabasesAsync(resourceGroupName, serverName, recommendedElasticPoolName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Returns information about an Azure SQL Database inside of an Azure
        /// Sql Recommended Elastic Pool.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Sql.IRecommendedElasticPoolOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the Resource Group to which the server
        /// belongs.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Database Server on which the
        /// database is hosted.
        /// </param>
        /// <param name='recommendedElasticPoolName'>
        /// Required. The name of the Azure SQL Recommended Elastic Pool to be
        /// retrieved.
        /// </param>
        /// <returns>
        /// Represents the response to a List Azure Sql Database request.
        /// </returns>
        public static Task<DatabaseListResponse> ListDatabasesAsync(this IRecommendedElasticPoolOperations operations, string resourceGroupName, string serverName, string recommendedElasticPoolName)
        {
            return operations.ListDatabasesAsync(resourceGroupName, serverName, recommendedElasticPoolName, CancellationToken.None);
        }
        
        /// <summary>
        /// Returns information about Azure SQL Recommended Elastic Pools.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Sql.IRecommendedElasticPoolOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the Resource Group to which the Azure SQL
        /// Recommended Serve belongs.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Database Server in which Azure
        /// SQL Recommended Elastic Pools are hosted.
        /// </param>
        /// <param name='expand'>
        /// Required. The comma separated list of child objects that we want to
        /// expand on in response.
        /// </param>
        /// <returns>
        /// Represents the response to a List Azure Sql Recommended Elastic
        /// Pool request.
        /// </returns>
        public static RecommendedElasticPoolListResponse ListExpanded(this IRecommendedElasticPoolOperations operations, string resourceGroupName, string serverName, string expand)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IRecommendedElasticPoolOperations)s).ListExpandedAsync(resourceGroupName, serverName, expand);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Returns information about Azure SQL Recommended Elastic Pools.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Sql.IRecommendedElasticPoolOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the Resource Group to which the Azure SQL
        /// Recommended Serve belongs.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Database Server in which Azure
        /// SQL Recommended Elastic Pools are hosted.
        /// </param>
        /// <param name='expand'>
        /// Required. The comma separated list of child objects that we want to
        /// expand on in response.
        /// </param>
        /// <returns>
        /// Represents the response to a List Azure Sql Recommended Elastic
        /// Pool request.
        /// </returns>
        public static Task<RecommendedElasticPoolListResponse> ListExpandedAsync(this IRecommendedElasticPoolOperations operations, string resourceGroupName, string serverName, string expand)
        {
            return operations.ListExpandedAsync(resourceGroupName, serverName, expand, CancellationToken.None);
        }
        
        /// <summary>
        /// Returns information about an recommended elastic pool metrics.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Sql.IRecommendedElasticPoolOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the Resource Group to which the server
        /// belongs.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Database Server.
        /// </param>
        /// <param name='recommendedElasticPoolName'>
        /// Required. The name of the Azure SQL Recommended Elastic Pool to be
        /// retrieved.
        /// </param>
        /// <returns>
        /// Represents the response to a List Azure Sql Recommended Elastic
        /// Pool metrics request.
        /// </returns>
        public static RecommendedElasticPoolListMetricsResponse ListMetrics(this IRecommendedElasticPoolOperations operations, string resourceGroupName, string serverName, string recommendedElasticPoolName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IRecommendedElasticPoolOperations)s).ListMetricsAsync(resourceGroupName, serverName, recommendedElasticPoolName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Returns information about an recommended elastic pool metrics.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Sql.IRecommendedElasticPoolOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the Resource Group to which the server
        /// belongs.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Database Server.
        /// </param>
        /// <param name='recommendedElasticPoolName'>
        /// Required. The name of the Azure SQL Recommended Elastic Pool to be
        /// retrieved.
        /// </param>
        /// <returns>
        /// Represents the response to a List Azure Sql Recommended Elastic
        /// Pool metrics request.
        /// </returns>
        public static Task<RecommendedElasticPoolListMetricsResponse> ListMetricsAsync(this IRecommendedElasticPoolOperations operations, string resourceGroupName, string serverName, string recommendedElasticPoolName)
        {
            return operations.ListMetricsAsync(resourceGroupName, serverName, recommendedElasticPoolName, CancellationToken.None);
        }
    }
}
