// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.Azure.Management.Sql;
using Microsoft.Azure.Management.Sql.Models;

namespace Microsoft.Azure.Management.Sql
{
    /// <summary>
    /// The Windows Azure SQL Database management API provides a RESTful set of
    /// web services that interact with Windows Azure SQL Database services to
    /// manage your databases. The API enables users to create, retrieve,
    /// update, and delete databases and servers.
    /// </summary>
    public static partial class RecommendedIndexOperationsExtensions
    {
        /// <summary>
        /// Returns details on recommended index.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Sql.IRecommendedIndexOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the Resource Group.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL server.
        /// </param>
        /// <param name='databaseName'>
        /// Required. The name of the Azure SQL database.
        /// </param>
        /// <param name='schemaName'>
        /// Required. The name of the Azure SQL database schema.
        /// </param>
        /// <param name='tableName'>
        /// Required. The name of the Azure SQL database table.
        /// </param>
        /// <param name='indexName'>
        /// Required. The name of the Azure SQL database recommended index.
        /// </param>
        /// <returns>
        /// Represents the response to a get recommended index request.
        /// </returns>
        public static RecommendedIndexGetResponse Get(this IRecommendedIndexOperations operations, string resourceGroupName, string serverName, string databaseName, string schemaName, string tableName, string indexName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IRecommendedIndexOperations)s).GetAsync(resourceGroupName, serverName, databaseName, schemaName, tableName, indexName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Returns details on recommended index.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Sql.IRecommendedIndexOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the Resource Group.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL server.
        /// </param>
        /// <param name='databaseName'>
        /// Required. The name of the Azure SQL database.
        /// </param>
        /// <param name='schemaName'>
        /// Required. The name of the Azure SQL database schema.
        /// </param>
        /// <param name='tableName'>
        /// Required. The name of the Azure SQL database table.
        /// </param>
        /// <param name='indexName'>
        /// Required. The name of the Azure SQL database recommended index.
        /// </param>
        /// <returns>
        /// Represents the response to a get recommended index request.
        /// </returns>
        public static Task<RecommendedIndexGetResponse> GetAsync(this IRecommendedIndexOperations operations, string resourceGroupName, string serverName, string databaseName, string schemaName, string tableName, string indexName)
        {
            return operations.GetAsync(resourceGroupName, serverName, databaseName, schemaName, tableName, indexName, CancellationToken.None);
        }
        
        /// <summary>
        /// We execute or cancel index operations by updating index state.
        /// Allowed state transitions are :Active          -> Pending
        /// - Start index creation processPending         -> Active
        /// - Cancel index creationActive/Pending  -> Ignored
        /// - Ignore index recommendation so it will no longer show
        /// in active recommendationsIgnored         -> Active           -
        /// Restore index recommendationSuccess         -> Pending Revert   -
        /// Revert index that has been createdPending Revert  -> Revert
        /// Canceled  - Cancel index revert operation
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Sql.IRecommendedIndexOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the Resource Group to which the Azure SQL
        /// Database Server belongs.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Database Server on which the
        /// database is hosted.
        /// </param>
        /// <param name='databaseName'>
        /// Required. The name of the Azure SQL Database.
        /// </param>
        /// <param name='schemaName'>
        /// Required. The name of the Azure SQL Database schema.
        /// </param>
        /// <param name='tableName'>
        /// Required. The name of the Azure SQL Database table.
        /// </param>
        /// <param name='indexName'>
        /// Required. The name of the Azure SQL Database recommended index.
        /// </param>
        /// <param name='parameters'>
        /// Required. The required parameters for updating index state.
        /// </param>
        /// <returns>
        /// Represents the response to a get recommended index request.
        /// </returns>
        public static RecommendedIndexUpdateResponse Update(this IRecommendedIndexOperations operations, string resourceGroupName, string serverName, string databaseName, string schemaName, string tableName, string indexName, RecommendedIndexUpdateParameters parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IRecommendedIndexOperations)s).UpdateAsync(resourceGroupName, serverName, databaseName, schemaName, tableName, indexName, parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// We execute or cancel index operations by updating index state.
        /// Allowed state transitions are :Active          -> Pending
        /// - Start index creation processPending         -> Active
        /// - Cancel index creationActive/Pending  -> Ignored
        /// - Ignore index recommendation so it will no longer show
        /// in active recommendationsIgnored         -> Active           -
        /// Restore index recommendationSuccess         -> Pending Revert   -
        /// Revert index that has been createdPending Revert  -> Revert
        /// Canceled  - Cancel index revert operation
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Sql.IRecommendedIndexOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the Resource Group to which the Azure SQL
        /// Database Server belongs.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Database Server on which the
        /// database is hosted.
        /// </param>
        /// <param name='databaseName'>
        /// Required. The name of the Azure SQL Database.
        /// </param>
        /// <param name='schemaName'>
        /// Required. The name of the Azure SQL Database schema.
        /// </param>
        /// <param name='tableName'>
        /// Required. The name of the Azure SQL Database table.
        /// </param>
        /// <param name='indexName'>
        /// Required. The name of the Azure SQL Database recommended index.
        /// </param>
        /// <param name='parameters'>
        /// Required. The required parameters for updating index state.
        /// </param>
        /// <returns>
        /// Represents the response to a get recommended index request.
        /// </returns>
        public static Task<RecommendedIndexUpdateResponse> UpdateAsync(this IRecommendedIndexOperations operations, string resourceGroupName, string serverName, string databaseName, string schemaName, string tableName, string indexName, RecommendedIndexUpdateParameters parameters)
        {
            return operations.UpdateAsync(resourceGroupName, serverName, databaseName, schemaName, tableName, indexName, parameters, CancellationToken.None);
        }
    }
}
