// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.Azure;
using Microsoft.WindowsAzure.Management.Sql;
using Microsoft.WindowsAzure.Management.Sql.Models;

namespace Microsoft.WindowsAzure.Management.Sql
{
    /// <summary>
    /// This is the main client class for interacting with the Azure SQL
    /// Database REST APIs.
    /// </summary>
    public static partial class ServerOperationsExtensions
    {
        /// <summary>
        /// Changes the administrative password of an existing Azure SQL
        /// Database Server for a given subscription.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Sql.IServerOperations.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Database Server that will have
        /// the administrator password changed.
        /// </param>
        /// <param name='parameters'>
        /// Required. The necessary parameters for modifying the adminstrator
        /// password for a server.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static AzureOperationResponse ChangeAdministratorPassword(this IServerOperations operations, string serverName, ServerChangeAdministratorPasswordParameters parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IServerOperations)s).ChangeAdministratorPasswordAsync(serverName, parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Changes the administrative password of an existing Azure SQL
        /// Database Server for a given subscription.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Sql.IServerOperations.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Database Server that will have
        /// the administrator password changed.
        /// </param>
        /// <param name='parameters'>
        /// Required. The necessary parameters for modifying the adminstrator
        /// password for a server.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<AzureOperationResponse> ChangeAdministratorPasswordAsync(this IServerOperations operations, string serverName, ServerChangeAdministratorPasswordParameters parameters)
        {
            return operations.ChangeAdministratorPasswordAsync(serverName, parameters, CancellationToken.None);
        }
        
        /// <summary>
        /// Provisions a new SQL Database server in a subscription.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Sql.IServerOperations.
        /// </param>
        /// <param name='parameters'>
        /// Required. The parameters needed to provision a server.
        /// </param>
        /// <returns>
        /// The response returned from the Create Server operation.  This
        /// contains all the information returned from the service when a
        /// server is created.
        /// </returns>
        public static ServerCreateResponse Create(this IServerOperations operations, ServerCreateParameters parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IServerOperations)s).CreateAsync(parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Provisions a new SQL Database server in a subscription.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Sql.IServerOperations.
        /// </param>
        /// <param name='parameters'>
        /// Required. The parameters needed to provision a server.
        /// </param>
        /// <returns>
        /// The response returned from the Create Server operation.  This
        /// contains all the information returned from the service when a
        /// server is created.
        /// </returns>
        public static Task<ServerCreateResponse> CreateAsync(this IServerOperations operations, ServerCreateParameters parameters)
        {
            return operations.CreateAsync(parameters, CancellationToken.None);
        }
        
        /// <summary>
        /// Deletes the specified Azure SQL Database Server from a subscription.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Sql.IServerOperations.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Database Server to be deleted.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static AzureOperationResponse Delete(this IServerOperations operations, string serverName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IServerOperations)s).DeleteAsync(serverName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Deletes the specified Azure SQL Database Server from a subscription.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Sql.IServerOperations.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Database Server to be deleted.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<AzureOperationResponse> DeleteAsync(this IServerOperations operations, string serverName)
        {
            return operations.DeleteAsync(serverName, CancellationToken.None);
        }
        
        /// <summary>
        /// Returns all SQL Database Servers that are provisioned for a
        /// subscription.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Sql.IServerOperations.
        /// </param>
        /// <returns>
        /// The response structure for the Server List operation.  Contains a
        /// list of all the servers in a subscription.
        /// </returns>
        public static ServerListResponse List(this IServerOperations operations)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IServerOperations)s).ListAsync();
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Returns all SQL Database Servers that are provisioned for a
        /// subscription.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Sql.IServerOperations.
        /// </param>
        /// <returns>
        /// The response structure for the Server List operation.  Contains a
        /// list of all the servers in a subscription.
        /// </returns>
        public static Task<ServerListResponse> ListAsync(this IServerOperations operations)
        {
            return operations.ListAsync(CancellationToken.None);
        }
    }
}
